package molpage.pri;

import org.hibernate.SessionFactory;
import java.io.File;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContext;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import sims.hibernate.*;

import org.hibernate.cfg.Configuration;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import molpage.com.Util;

public class WebappConfig {

private static File protocolDirectory = null;
private static File dataDirectory = null;
private static File workDirectory = null;
private static boolean isFtp = false;
private static File ftpDirectory = null;
private static String ftpServerHint = null;
private static String ftpDirectoryHint = null;
private static String logoutProc = null;
private static String secondUser = null;
private static SessionFactory sampleFactory = null;
private static int viewLimit = 1000;
private static Configuration sampleCfg = null;
private static File systemConstants = null;

public static void destroy(){
  try {
    if (sampleFactory != null) {
      sampleFactory.close();
      sampleFactory = null;
    }
    Util.writeEndOfApplication();
  }
  catch (Exception ex) {
    Util.writeLog("Close factory",
                   ex.getMessage(),
                   "");
  }
}

public static  void contextInitialized(ServletContextEvent sctx)
  {
    contextInitialized( sctx.getServletContext());
  }
public static  void contextInitialized(ServletContext sct)
  {
	File confDirectory = new File( sct.getRealPath("/conf"));
    confDirectory.mkdir();
    systemConstants = new File( confDirectory, "systemConst.xml");
    File logDirectory = new File( sct.getRealPath("/log"));
    logDirectory.mkdir();
    Util.initializeLog( logDirectory);
//    VocabulName.init();
    checkConstants(systemConstants);
    VocabulName.processConstants(WebappConfig.getSystemConstants());
    sampleCfg = new Configuration();
    String url = sct.getInitParameter("sample.url");
    String username = sct.getInitParameter("sample.username");
    String password = sct.getInitParameter("sample.password");
    if (password != null) sampleCfg.setProperty("hibernate.connection.password",
                                          password);
    if (username != null) sampleCfg.setProperty("hibernate.connection.username",
                                          username);
    if (url != null) sampleCfg.setProperty("hibernate.connection.url", url);
    try {
       sampleCfg.addClass(User.class)
      .addClass(STechnologyAccess.class)
      .addClass(STechnology.class)
      .addClass(UserRights.class)
      .addClass(UserGroup.class)
      .addClass(UserGroupRights.class)
      .addClass(UserGroupMembership.class)
      .addClass(VersionNumber.class)
      .addClass(ProjectPartner.class)

      .addClass(DataType.class)
      .addClass(ProtocolType.class)
      .addClass(PendingTask.class)

      .addClass(Top.class)
      .addClass(Middle.class)
      .addClass(Bottom.class)
      .addClass(Relation.class)
      .addClass(RelationType.class)
      .addClass(TopMetaData.class)
      .addClass(MiddleMetaData.class)
      .addClass(BottomMetaData.class);
    }
    catch (Exception ex) {
      Util.writeLog("Start application - add class to factory:",
                     ex.getMessage(),
                     "");
    }

   for (int i = 1; i <= VocabulName.getVocabularyCount() ; i++) {
      try {
        Class cvtxy = Class.forName(VocabulName.getClassNameVT(i));
        sampleCfg.addClass(cvtxy);
      } catch (Exception ex) {
            Util.writeLog("Start application - add class to factory:",
                           ex.getMessage(),
                           VocabulName.getClassNameVT(i));
      }
    }
    for (int i = 1; i <= VocabulName.getDependencyCount() ; i++) {
      try {
        Class cvtxy = VocabulName.getDependencyClass(i);
        sampleCfg.addClass(cvtxy);
      } catch (Exception ex) {
          Util.writeLog("Start application - add class to factory:",
                         ex.getMessage(),
                         VocabulName.getDependencyClassName(i));
      }
    }
    for (int i = 1; i <= VocabulName.getIdentificyCount() ; i++) {
      try {
        Class cvtxy = VocabulName.getIdentificyClass(i);
        sampleCfg.addClass(cvtxy);
      } catch (Exception ex) {
          Util.writeLog("Start application - add class to factory:",
                         ex.getMessage(),
                         VocabulName.getIdentificyClassName(i));
      }
    }
    try {
      sampleFactory = sampleCfg.buildSessionFactory();
    }
    catch (Exception ex) {
      Util.writeLog("Build factory error", ex.getMessage(), "");
    }  
      // Prepares metadata
    String tmp11 = sct.getInitParameter("data.directory");
    if (tmp11 != null) {
      dataDirectory = new File( tmp11);
      dataDirectory.mkdir();
    }
    if (dataDirectory == null) {
      dataDirectory = new File(sct.getRealPath("/data_files"));
      dataDirectory.mkdir();
    }
    tmp11 = sct.getInitParameter("protocol.directory");
    if (tmp11 != null) {
      protocolDirectory = new File( tmp11);
      protocolDirectory.mkdir();
    }
    if (protocolDirectory == null) {
      protocolDirectory = new File(sct.getRealPath("/protocol_files"));
      protocolDirectory.mkdir();
    }
    workDirectory = new File( sct.getRealPath("/work"));
    workDirectory.mkdir();

    // Prepares limit of view tables row number
    tmp11 = sct.getInitParameter("viewlimit");
    if (tmp11 != null) {
        viewLimit = Util.getInt( tmp11);
    }
    tmp11 = sct.getInitParameter("ftp.directory");
    if (tmp11 != null) {
      ftpDirectory = new File( tmp11);
      isFtp = (ftpDirectory != null);
    }
    ftpServerHint = sct.getInitParameter("ftp.server.popup");
    ftpDirectoryHint = sct.getInitParameter("ftp.directory.popup");
    logoutProc = sct.getInitParameter("logoutproc");
    secondUser = sct.getInitParameter("seconduser");
  }

  public static String getDataDirectoryPath(){
    if (dataDirectory==null)return "not exists";
    else
      return dataDirectory.getAbsolutePath();
  }

  public static String getProtocolDirectoryPath(){
    if (protocolDirectory==null)return "not exists";
    else
      return protocolDirectory.getAbsolutePath();
  }
  public static File getProtocolDirectory(){
      return protocolDirectory;
  }
  public static File getDataDirectory(){
      return dataDirectory;
  }
  public static File getWorkDirectory(){
      return workDirectory;
  }
  public static boolean isFtp(){
      return isFtp;
  }
  public static File getFtpDirectory(){
      return ftpDirectory;
  }
  public static String getFtpDirectoryPath(){
    if (ftpDirectory==null)return "not exists";
    else
      return ftpDirectory.getAbsolutePath();
  }
  public static String getFtpServerHint(){
      return ftpServerHint;
  }
  public static String getFtpDirectoryHint(){
      return ftpDirectoryHint;
  }
  public static String getLogoutProc(){
      return logoutProc;
  }
  public static String getSecondUser(){
      return secondUser;
  }
  public static int getViewLimit(){
      return viewLimit;
  }
  public static SessionFactory getSamplesSessionFactory(){
      if (sampleFactory == null)
        sampleFactory = sampleCfg.buildSessionFactory();
      return sampleFactory;
  }
  public static void setSecondUser( String val){
      secondUser = val;
  }

  public static File getSystemConstants(){
      return systemConstants;
  }
  
//--------------------------------------------------
  private static int getNumber( String str) throws Exception {
		int result = 0;
		if (str!=null){
		  int n = MyUtil.getIntegerValue(str);
		  if (n<0||n>79) 
			Util.writeAccessLog("System constants:","Invalid number",str);
		  else result = n;
		}
		return result;
	  }
  private static int processMainNames( Element root) throws Exception {
	   int result = 0; String mainName; int attrCount;
	   NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XBottom);
	   int k = nodeLst1.getLength();
	   if (k != 1){ result = 1;
	     Util.writeAccessLog("System constants:","Invalid Bottom name","");
	   }
	   else {
	     Element bottom = (Element) nodeLst1.item(0);
	     mainName = bottom.getAttribute(XMLTags.Xplural);
	     mainName = bottom.getAttribute(XMLTags.Xsingular);
	     mainName = bottom.getAttribute(XMLTags.Xprefix);
	     attrCount = getNumber(bottom.getAttribute(XMLTags.XstringNumb));
	     attrCount = getNumber(bottom.getAttribute(XMLTags.XintegerNumb));
	     attrCount = getNumber(bottom.getAttribute(XMLTags.XfloatNumb));
	     attrCount = getNumber(bottom.getAttribute(XMLTags.XdateNumb));
	   }
	   nodeLst1 = root.getElementsByTagName(XMLTags.XMiddle);
	   k = nodeLst1.getLength();
	   if (k != 1){ result = 1;
	     Util.writeAccessLog("System constants:","Invalid Middle name","");
	   }
	   else {
	     Element middle = (Element) nodeLst1.item(0);
	     mainName = middle.getAttribute(XMLTags.Xplural);
	     mainName = middle.getAttribute(XMLTags.Xsingular);
	     mainName = middle.getAttribute(XMLTags.Xprefix);
	     attrCount = getNumber(middle.getAttribute(XMLTags.XstringNumb));
	     attrCount = getNumber(middle.getAttribute(XMLTags.XintegerNumb));
	     attrCount = getNumber(middle.getAttribute(XMLTags.XfloatNumb));
	     attrCount = getNumber(middle.getAttribute(XMLTags.XdateNumb));
	   }
	   nodeLst1 = root.getElementsByTagName(XMLTags.XTop);
	   k = nodeLst1.getLength();
	   if (k != 1){ result = 1;
	     Util.writeAccessLog("System constants:","Invalid Top name","");
	   }
	   else {
	     Element top = (Element) nodeLst1.item(0);
	     mainName = top.getAttribute(XMLTags.Xplural);
	     mainName = top.getAttribute(XMLTags.Xsingular);
	     mainName = top.getAttribute(XMLTags.Xprefix);
	     attrCount = getNumber(top.getAttribute(XMLTags.XstringNumb));
	     attrCount = getNumber(top.getAttribute(XMLTags.XintegerNumb));
	     attrCount = getNumber(top.getAttribute(XMLTags.XfloatNumb));
	     attrCount = getNumber(top.getAttribute(XMLTags.XdateNumb));
	   }
	   return result;
	  }
	  private static int processUploadNames( Element root) throws Exception {
	    int result = 0; int myUplCount;
	    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XUploadName);
	    myUplCount = nodeLst1.getLength();
	    if (myUplCount < 1){ result = 1;
	      Util.writeAccessLog("System constants:","Too few uploadname","");
	    }
	    else {
	      for (int i = 0; i < myUplCount; i++) {
	        Element group = (Element) nodeLst1.item(i);
	        String columnname = group.getAttribute(XMLTags.Xcolumn);
	        int m = Fieldname.getRowNumber(columnname);
	        if (m < 0) {
	          Util.writeAccessLog("System constants:","Invalid column name",columnname);
	          result = 2;
	        }
	      }
	    }
	    return result;
	  }
	  
	  private static int processVocabularies( Element root) throws Exception {
	    int result = 0; int vmax = 0;
	    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XVocabulary);
		int myCount = nodeLst1.getLength();
		if (myCount < 1){ result = 1;
		  Util.writeAccessLog("System constants:","Too few vocabularies","");
		}
		else {
		  for (int i = 0; i < myCount; i++) {
		    Element group = (Element) nodeLst1.item(i);
		    String numb = group.getAttribute(XMLTags.Xseq_numb);
		    String vocabularyName = group.getAttribute(XMLTags.Xtable);	
		      vocabularyName = group.getAttribute(XMLTags.Xitem);
		    int n = MyUtil.getIntegerValue(numb);
		    if (n<1||n>79){ result = 1;
			  Util.writeAccessLog("System constants-Vocabulary:","Invalid number",group.toString());
		    } else if (n>vmax)vmax = n;
		  }
		}
	    return result;
	  }
	  private static int processIdentificies( Element root) throws Exception {
	    int result = 0; int vmax = 0;
	    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XIdentificy);
		int myCount = nodeLst1.getLength();
		if (myCount < 1){ result = 1;
		  Util.writeAccessLog("System constants:","Too few identificies","");
		}
		else {
		  for (int i = 0; i < myCount; i++) {
		    Element group = (Element) nodeLst1.item(i);
		    String numb = group.getAttribute(XMLTags.Xseq_numb);
		    String identificyName = group.getAttribute(XMLTags.Xtable);	
		      identificyName = group.getAttribute(XMLTags.Xitem);
	        int n = MyUtil.getIntegerValue(numb);
		    if (n<1||n>9){ result = 1;
			  Util.writeAccessLog("System constants-Identificy:","Invalid number",group.toString());
		    } else if (n>vmax)vmax = n;
		  }
		}
		return result;
	  }
	  private static int processDependencies( Element root) throws Exception {
	    int result = 0; int vmax = 0;
	    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XDependency);
		int myCount = nodeLst1.getLength();
		if (myCount < 1){ result = 1;
		  Util.writeAccessLog("System constants:","Too few dependencies","");
		}
		else {
		  for (int i = 0; i < myCount; i++) {
		    Element group = (Element) nodeLst1.item(i);
		    String numb = group.getAttribute(XMLTags.Xseq_numb);
		    String dependencyName = group.getAttribute(XMLTags.Xtable);	
		      dependencyName = group.getAttribute(XMLTags.Xindependent);
		      dependencyName = group.getAttribute(XMLTags.Xdependent);
		      dependencyName = group.getAttribute(XMLTags.XindependentClass);
		      dependencyName = group.getAttribute(XMLTags.XdependentClass);
	        int n = MyUtil.getIntegerValue(numb);
		    if (n<1||n>9){ result = 1;
			  Util.writeAccessLog("System constants-Dependency:","Invalid number",group.toString());
		    } else if (n>vmax)vmax = n;
		  }
		}
		return result;
	  }
	  private static int processIdentifDetails( int level, Element bottom) throws Exception {
	    int result = 0;
/*	    
	    for (int i = 0; i < 4; i++){
	      String param = Util.getParameterName0("s", i+1);
	      String delim = bottom.getAttribute(param);
	      levelIdDelimiter[level][i] = (delim==null)?"":delim;
	    }
	    levelIdCount[level]=0;
	    for (int i = 0; i < 3; i++){
	      String param = Util.getParameterName0("ivt", i+1);
	      String ivt = bottom.getAttribute(param);
	      int ivtNum = MyUtil.getIntegerValue(ivt);
	      if (ivtNum<1 || ivtNum>identificyCount) break;
	      levelIdNum[level][i] = ivtNum;
	      levelIdCount[level]++;
	    }
	    String numb = bottom.getAttribute("ptype");
	    int ptype = MyUtil.getIntegerValue(numb);
	    levelPostfixType[level] = (ptype==2)?2:1;
*/	    
		return result;
	  }
	  private static int processIdStructure( Element root) throws Exception {
	    int result = 0; 
	    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XBottom);
	    int num = nodeLst1.getLength();
	    if ( num == 1) processIdentifDetails( 2, (Element)nodeLst1.item(0));
	    else Util.writeAccessLog("System constants:","BottomIds not processed","");
	    nodeLst1 = root.getElementsByTagName(XMLTags.XMiddle);
	    num = nodeLst1.getLength();
	    if ( num == 1) processIdentifDetails( 1, (Element)nodeLst1.item(0));
	    else Util.writeAccessLog("System constants:","MiddleIds not processed","");
	    nodeLst1 = root.getElementsByTagName(XMLTags.XTop);
	    num = nodeLst1.getLength();
	    if ( num == 1) processIdentifDetails( 0, (Element)nodeLst1.item(0));
	    else Util.writeAccessLog("System constants:","TopIds not processed","");
		return result;
	  }
	  private static int processRoot( Element root) throws Exception{
	    int result = 0;
	    String version = root.getAttribute(XMLTags.Xversion);
	    if (version==null||!version.equals("1.17")){
	      Util.writeAccessLog("System constants:","Invalid version","");
	      return 1;
	    }
	    NodeList nodeLst1 = root.getElementsByTagName(XMLTags.XUploadNames);
	    int num = nodeLst1.getLength();
	    if ( num != 1){
	      Util.writeAccessLog("System constants:","Invalid upload names","");
	      return 1;
	    }
	    result = processUploadNames( (Element)nodeLst1.item(0));
	    nodeLst1 = root.getElementsByTagName(XMLTags.XMainNames);
		num = nodeLst1.getLength();
	    if ( num != 1){
	      Util.writeAccessLog("System constants:","Invalid main names","");
	      return 1;
	    }
	    result = processMainNames( (Element)nodeLst1.item(0));
	    nodeLst1 = root.getElementsByTagName(XMLTags.XVocabularies);
		num = nodeLst1.getLength();
	    if (num==1) processVocabularies( (Element)nodeLst1.item(0));
	    else Util.writeAccessLog("System constants:","Vocabularies not defined","");
	    nodeLst1 = root.getElementsByTagName(XMLTags.XIdentificies);
		num = nodeLst1.getLength();
	    if (num==1) processIdentificies( (Element)nodeLst1.item(0));
	    else Util.writeAccessLog("System constants:","Identificies not defined","");
	    nodeLst1 = root.getElementsByTagName(XMLTags.XDependencies);
		num = nodeLst1.getLength();
	    if (num==1) processDependencies( (Element)nodeLst1.item(0));
	    else Util.writeAccessLog("System constants:","Dependencies not defined","");
	    nodeLst1 = root.getElementsByTagName(XMLTags.XIdStructure);
		num = nodeLst1.getLength();
	    if (num==1) processIdStructure( (Element)nodeLst1.item(0));
	    else Util.writeAccessLog("System constants:","IdStructure not defined","");
	    return result;
	  }

	  public static int checkConstants( File systemConst){
	    int result = 0;
	    try {
	      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
	      DocumentBuilder db = dbf.newDocumentBuilder();
	      Document doc = db.parse(systemConst);
	      Element root = doc.getDocumentElement();
	      root.normalize();
	      result = processRoot( root);
	    }
	    catch (Exception e) {
	      if (e!=null){
	      java.io.CharArrayWriter cw = new java.io.CharArrayWriter();
	      java.io.PrintWriter pw = new java.io.PrintWriter(cw,true);
	      e.printStackTrace(pw);
	      Util.writeAccessLog(cw.toString(),"","");
	     }
	     result = 1;
	    }
	    return result;
	  }

//	--------------------------------------------------
  
}
