package sims.hibernate;

public final class TempMetaData {
  private static String thisName = "sims.hibernate.";
    private String visibleName;
    private String shortName;
    private String longName;
    private String uploadName;
    private String setMethodName;
    private int viewColumnNumber;
    private boolean sortable;
    private int valueRowNumber;
    private byte actualColumnNumber;
//    private boolean canShowInReport;
    private boolean required = false;
    private int level;
    private int baseType;
    private Class baseClass;
    private int tempColumnNumber;//used by batch upload
    private int positionInFilter;
    private boolean userDef;
    
    public boolean isUserDef(){
      return userDef;
    }
    public boolean isRequired(){
      return required;
    }
    public int getPositionInFilter(){
        return positionInFilter;
      }
    public int getTempColumnNumber(){
      return tempColumnNumber;
    }
    public void setTempColumnNumber( int val){
      tempColumnNumber = val;
    }

    public TempMetaData(int level, MetaData metadata) {
      valueRowNumber = -1; 	
      visibleName = metadata.getVisibleName();
      shortName = metadata.getShortName();
      longName = metadata.getLongName();
      uploadName = metadata.getUploadNameCom();
      if (uploadName==null) uploadName = "";
      viewColumnNumber = metadata.getViewColumnNumber().intValue();
      sortable = metadata.isSortable();
      actualColumnNumber = 0;
      required = metadata.isRequiredCom();
      positionInFilter = metadata.getPositionInFilterCom().intValue();
      this.level = level;
      baseType = VocabulName.getNameType(visibleName);
      if (baseType==TypeName.TABLE || baseType==TypeName.IDENTIFICY)
        try {
          String clasName = visibleName.toUpperCase();
          baseClass = Class.forName(thisName + clasName);
        } catch (ClassNotFoundException ex) {
//          Util.writeLog("Class not exists", "", ex.getMessage());
          baseType=TypeName.BAD_TYPE;
        }
      setMethodName = "set" + Character.toUpperCase(visibleName.charAt(0))
                      + visibleName.substring(1);
      userDef = true;
    }
/*    
    public TempMetaData(int level, String visname, String upname) {
      valueRowNumber = -1; 	
      visibleName = visname;
      shortName = "";  
      longName = null;
      uploadName = upname;
      viewColumnNumber = 0;
      sortable = false;
      actualColumnNumber = 0;
      required = false;
      positionInFilter = 0;
      this.level = level;
      baseType = 0;
      setMethodName = "set" + Character.toUpperCase(visibleName.charAt(0))
                        + visibleName.substring(1);
      userDef = false;
    }
    public void completeData( MetaData metadata) {
      shortName = metadata.getShortName();
      longName = metadata.getLongName();
      uploadName = metadata.getUploadName();
      if (uploadName==null||uploadName.equals("")) uploadName = longName;
      viewColumnNumber = metadata.getViewColumnNumber().intValue();
      sortable = metadata.isSortable();
      positionInFilter = metadata.getVisiblePositionInFilter().intValue();
      userDef = true;
    }
*/    
    public String getSetMethodName(){
      return setMethodName;
    }
    public Class getBaseClass() {
      return baseClass;
    }
    public int getBaseType() {
      return baseType;
    }
    public int getLevel() {
      return level;
    }
    public boolean isTable() {
      return baseType==TypeName.TABLE;
    }
    public boolean isIdentificy() {
      return baseType==TypeName.IDENTIFICY;
    }
    public boolean isTopLevel() {
      return level == 0;
    }
    public boolean isMiddleLevel() {
      return level == 1;
    }
    public boolean isBottomLevel() {
      return level == 2;
    }
    public String getVisibleName() {
            return visibleName;
    }

    public String getShortName() {
            return shortName;
    }

    public String getLongName() {
            return longName;
    }

    public String getUploadName() {
            return uploadName;
    }

    public int getViewColumnNumber() {
            return viewColumnNumber;
    }

    public byte getActualColumnNumber() {
            return actualColumnNumber;
    }

    public int getValueRowNumber() {
            return valueRowNumber;
    }
/*
    public boolean getSortable() {
            return sortable;
    }
*/
    public boolean isSortable() {
            return sortable;
    }

    public void setShortName(String val) {
            shortName = val;
    }

    public void setLongName(String val) {
            longName = val;
    }

    public void setUploadName(String val) {
            uploadName = val;
    }

    public void setViewColumnNumber(int val) {
            viewColumnNumber = val;
    }

    public void setActualColumnNumber(byte val) {
            actualColumnNumber = val;
    }

    public void setValueRowNumber(int cNumber) {
            valueRowNumber = cNumber;
    }
/*
    public void setSortable(boolean val) {
            sortable = val;
    }
*/    
}

