package sims.hibernate;

import java.util.Stack;
import org.xml.sax.InputSource;
import java.io.StringReader;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public final class User extends Persistent {

  private String loginName;
  private String password;
// -------------------------------------------
// -------------------------------------------
  private Integer adminRights;
  private Integer exportRights;
  private String idPrefix;
  private String ftpDirectory;
  private Integer ftpIsAllowed;
  private String userSettings;
  private ProjectPartner projectPartner;
  private UserGroup defaultUserGroup;
  private String currentTechn;
// -------------------------------------------
// -------------------------------------------
  private Integer startWithWelcomePage;
  private String filterSettings; 
//-------------------------------------------
  private int viewLimit = 0;
  private boolean showFullId = false;

// -------------------------------------------
  public String getCurrentTechn(){
	return currentTechn;  
  }
  public void setCurrentTechn( String val){
	currentTechn = val;
  }
// -------------------------------------------
  public boolean isWithWelcomePageStart() {
    return startWithWelcomePage!=null && startWithWelcomePage.byteValue() > 0;
  }
  public Integer getStartWithWelcomePage() {
    return startWithWelcomePage;
  }
  public String getFilterSettings() {
    return filterSettings;
  }
  public String getLoginName() {
    return loginName;
  }
  public String getPassword() {
    return password;
  }
  public ProjectPartner getProjectPartner() {
	return projectPartner;
  }
// -------------------------------------------
// -------------------------------------------
  public void setStartWithWelcomePage(Integer val) {
    startWithWelcomePage = val;
  }
  public void setFilterSettings(String val) {
    filterSettings = val;
  }
  public void setLoginName(String string) {
    loginName = string;
  }
  public void setPassword(String string) {
    password = string;
  }
  public void setProjectPartner(ProjectPartner val) {
    projectPartner = val;
  }
  public String toString() {
    return this.getId() + "  " + getVisibleName() + " : " + loginName;
  }
//-------------------------------------------
//-------------------------------------------
  public boolean isPowerAdministrator() {
    return (adminRights!=null)&&(adminRights.intValue() >= 3000);
  }
  public boolean isWriteAdministrator() {
    return (adminRights!=null)&&(adminRights.intValue() >= 2000);
  }
  public boolean isAdministrator() {
    return (adminRights!=null)&&(adminRights.intValue() >= 1000);
  }
  public UserGroup getDefaultUserGroup() {
    return defaultUserGroup;
  }
//-------------------------------------------
  public int getViewLimit() {
    return viewLimit;
  }
  public boolean isShowFullId() {
    return showFullId;
  }
  public void setViewLimit( int val) {
    viewLimit = val;
  }
  public void setShowFullId(boolean val) {
    showFullId = val;
  }
// -------------------------------------------
//  private static final String Xname = "name";
//  private static final String Xcomment = "comment";
  private static final String Xsettings = "User_Settings";
  private static final String Xversion = "version";
  private static final String XUserConf = "UserConf";
  private static final String Xviewlimit = "view_limit";
  private static final String XshowFullId = "show_full_id";
  private static final String XcurrTechn = "current_techn";

  public void importSettings( int defViewLimit, boolean defShowFullId){// throws Exception {
    viewLimit = defViewLimit; showFullId = defShowFullId;
    currentTechn = null;
    if (userSettings!=null && !userSettings.equals(""))
    try {
      InputSource is = new InputSource( new StringReader(userSettings));
      DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
      DocumentBuilder db = dbf.newDocumentBuilder();
      Document doc = db.parse(is);
      Element root = doc.getDocumentElement();
      root.normalize();
//      result = processRoot( root);
      String version = root.getAttribute(Xversion);
      if (version==null||!version.equals("1.17")){
//        errorlist.add("Invalid version");
        return;
      }
      NodeList nodeLst1 = root.getElementsByTagName(XUserConf);
      int num = nodeLst1.getLength();
      if ( num != 1){
//        errorlist.add("Invalid parameter groups");
        return;
      }
      Element userConf = (Element)nodeLst1.item(0);

      String vl = userConf.getAttribute(Xviewlimit);
      viewLimit = MyUtil.getIntegerValue(vl);
      String sfi = userConf.getAttribute(XshowFullId);
      showFullId = (sfi.equals("1"));
      currentTechn = userConf.getAttribute( XcurrTechn);
    }
    catch (Exception e) {
    }
  }

  private Stack<String>tagStack;
  private boolean isStarted = false;
  private StringBuffer jspOut;
  private String prevName;

  private void startTag( String name) {
    if (isStarted){
      jspOut.append('>');
      tagStack.push(prevName);
    }
    jspOut.append("\r\n");
    jspOut.append('<');
    jspOut.append(name);
    prevName = name;
    isStarted = true;
  }
  private void endTag() {
    if (isStarted){
      jspOut.append("/>");
      isStarted = false;
    } else {
      jspOut.append("\r\n");
      prevName = (String)tagStack.pop();
      jspOut.append("</");
      jspOut.append(prevName);
      jspOut.append('>');
    }
  }
  private void attribute(String name, String value) {
    jspOut.append(' ');
    jspOut.append(name);
    jspOut.append("=\"");
    jspOut.append(value);
    jspOut.append('\"');
  }

  public void exportSettings() {
    jspOut = new StringBuffer();
    tagStack = new Stack<String>();
    isStarted = false;
    startTag(Xsettings);
      attribute(Xversion,"1.17");
      startTag(XUserConf);
        attribute(Xviewlimit, String.valueOf( viewLimit));
        attribute(XshowFullId, isShowFullId() ? "1" : "0");
        if (currentTechn!=null) attribute(XcurrTechn, currentTechn);  
      endTag(); //User conf
    endTag();//User settings
    userSettings = jspOut.toString();
  }

// -------------------------------------------
  public boolean isEqual( User otherUser){
    return otherUser!=null && getId().equals(otherUser.getId());
  }
  public Integer getFtpIsAllowed() {
    return ftpIsAllowed;
  }
  public boolean isFtpAllowed() {
    return ftpIsAllowed!=null && ftpIsAllowed.byteValue() > 0;
  }

  public String getFtpDirectory() {
    return ftpDirectory;
  }
  public String getIdPrefix() {
    return idPrefix;
  }
  public Integer getExportRights() {
    return exportRights;
  }

  public boolean hasAdvancedExportRights() {
    return exportRights!=null && exportRights.byteValue() > 0;
  }

  public Integer getAdminRights() {
    return adminRights;
  }

  public String getUserSettings() {
    return userSettings;
  }
// -------------------------------------------
// -------------------------------------------
  public void setFtpIsAllowed(Integer val) {
    ftpIsAllowed = val;
  }
  public void setFtpDirectory(String val) {
    ftpDirectory = val;
  }
  public void setIdPrefix(String val) {
    idPrefix = val;
  }
  public void setDefaultUserGroup(UserGroup val) {
    defaultUserGroup = val;
  }
  public void setAdminRights(Integer val) {
    adminRights = val;
  }
  public void setExportRights(Integer val) {
    exportRights = val;
  }

  public void setUserSettings( String val) {
    userSettings = val;
  }
  private static final String[] accessVariants = {
      "non-accessible",
      "view only",
      "limited editing",
      "full editing"
  };
  private static final int[] accessValues = {
      0, 1000, 2000, 3000
  };
  public static String getAdminRightSelectionNew(){
    StringBuffer xyz = new StringBuffer();
    for (int i = 0; i < accessVariants.length; i++){
        xyz.append("<option ");
        xyz.append("value=\"")
            .append( accessValues[i])
            .append("\">")
            .append(accessVariants[i]);
      }
    return xyz.toString();
  }

  public String getAdminRightSelection(){
    StringBuffer xyz = new StringBuffer();
    int j = 1; int v = (adminRights!=null)?adminRights.intValue():0;
    while ( j < accessValues.length && v >= accessValues[j]) j++;
    j--;
    for (int i = 0; i < accessVariants.length; i++){
        xyz.append("<option ");
        if ( i==j) xyz.append("selected ");
        xyz.append("value=\"")
            .append( accessValues[i])
            .append("\">")
            .append(accessVariants[i]);
      }
    return xyz.toString();
  }
  public String getAdminRightText(){
    int j = 1; int v = (adminRights!=null)?adminRights.intValue():0;
    while ( j < accessValues.length && v >= accessValues[j]) j++;
    return accessVariants[--j];
  }
  public static String getAccessRightSelectionNew(){
    StringBuffer xyz = new StringBuffer();
    for (int i = 0; i < accessVariants.length; i++){
        xyz.append("<option ");
        xyz.append("value=\"")
            .append( accessValues[i])
            .append("\">")
            .append(accessVariants[i]);
      }
    return xyz.toString();
  }

}
